(function() {
    if (window.GaodunJSBridge) {
        return;
    }

    var messagingIframe;
    var bizMessagingIframe;
    var sendMessageQueue = [];

    var CUSTOM_PROTOCOL_SCHEME = 'gdbridge';
    var QUEUE_HAS_MESSAGE = '__queue_message__/';

    var responseCallbacks = {};
    var responseSuccessCallbacks = {};
    var responseFailureCallbacks = {};
    var responseCompleteCallbacks = {};
    var uniqueId = 1;

    function _createQueueReadyIframe(doc) {
        if (messagingIframe == null || messagingIframe == undefined) {
            messagingIframe = doc.createElement('iframe');
            messagingIframe.style.display = 'none';
            doc.documentElement.appendChild(messagingIframe);
        }
    }

    function _createQueueReadyIframe4biz(doc) {
        if (bizMessagingIframe == null || bizMessagingIframe == undefined) {
            bizMessagingIframe = doc.createElement('iframe');
            bizMessagingIframe.style.display = 'none';
            doc.documentElement.appendChild(bizMessagingIframe);
        }
    }

    function call(object) {
        var responseCallback = {
            success: object.success,
            fail: object.fail,
            complete: object.complete
        };

        var objectHandlerName = object.handlerName;

        delete object.handlerName;
        delete object.success;
        delete object.fail;
        delete object.complete;

        _doSend({handlerName: objectHandlerName, data: object}, responseCallback);
    }

    function _doSend(object, responseCallback) {
        var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
        if(responseCallback){
            responseSuccessCallbacks[callbackId] = responseCallback.success;
            responseFailureCallbacks[callbackId] = responseCallback.fail;
            responseCompleteCallbacks[callbackId] = responseCallback.complete;
        }

        object.callbackId = callbackId;

        sendMessageQueue.push(object);
        _createQueueReadyIframe(document);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        if (messageQueueString !== '[]') {
            _createQueueReadyIframe4biz(document);
            bizMessagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
        }
    }

    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function() {
            var message = JSON.parse(messageJSON);
            var responseSuccessCallback;
            var responseFailureCallback;
            var responseCompleteCallback;
            if (message.responseId) {
                var responseData = JSON.parse(message.responseData);
                if(responseData.status === 'success'){
                    responseSuccessCallback = responseSuccessCallbacks[message.responseId];
                    if(responseSuccessCallback){
                        responseSuccessCallback(responseData.result);
                    }
                }

                if(responseData.status === 'failure'){
                    responseFailureCallback = responseFailureCallbacks[message.responseId];
                    if(responseFailureCallback){
                       responseFailureCallback(responseData.result);
                    }
                }
                responseCompleteCallback = responseCompleteCallbacks[message.responseId];
                if(responseCompleteCallback){
                    responseCompleteCallback(responseData.complete);
                }

                delete responseSuccessCallbacks[message.responseId];
                delete responseFailureCallbacks[message.responseId];
                delete responseCompleteCallbacks[message.responseId];

            } else {
                var eventData = JSON.parse(message.data);
                var event = new MessageEvent(message.handlerName, { 'data': eventData});
                document.dispatchEvent(event);
            }
        });
    }

    function _handleMessageFromNative(messageJSON) {
        _dispatchMessageFromNative(messageJSON);
    }

    var GaodunJSBridge = window.GaodunJSBridge = {
        call: call,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('GaodunJSBridgeReady');
    readyEvent.bridge = GaodunJSBridge;
    doc.dispatchEvent(readyEvent);

})();